@@ -1367,9 +1367,32 @@ QQuick3DLightmapBaker *QQuick3DViewport::lightmapBaker()
 */
 void QQuick3DViewport::bakeLightmap()
 {
-    lightmapBaker()->bake();
+    QQuick3DSceneRenderer *renderer = getRenderer();
+    if (!renderer || !renderer->m_layer->renderData)
+        return;
+
+    const bool currentlyBaking = renderer->m_layer->renderData->lightmapBaker != nullptr;
+
+    if (!currentlyBaking)
+        lightmapBaker()->bake();
 }
 
+/*!
+    \internal
+*/
+void QQuick3DViewport::denoiseLightmap()
+{
+    QQuick3DSceneRenderer *renderer = getRenderer();
+    if (!renderer || !renderer->m_layer->renderData)
+        return;
+
+    const bool currentlyBaking = renderer->m_layer->renderData->lightmapBaker != nullptr;
+
+    if (!currentlyBaking)
+        lightmapBaker()->denoise();
+}
+
+
 void QQuick3DViewport::setGlobalPickingEnabled(bool isEnabled)
 {
     QQuick3DSceneRenderer *renderer = getRenderer();
@@ -1985,6 +2008,7 @@ QQuick3DPickResult QQuick3DViewport::processPickResult(const QSSGRenderPickResul
                                   pickResult.m_scenePosition,
                                   pickResult.m_localPosition,
                                   pickResult.m_faceNormal,
+                                  pickResult.m_sceneNormal,
                                   pickResult.m_instanceIndex);
 
     QQuick3DItem2D *frontend2DItem = qobject_cast<QQuick3DItem2D *>(frontendObject);
@@ -2086,6 +2110,13 @@ void QQuick3DViewport::updateCameraForLayer(const QQuick3DViewport &view3D, QSSG
     } else if (view3D.camera()) {
         layerNode.explicitCameras.append(static_cast<QSSGRenderCamera *>(QQuick3DObjectPrivate::get(view3D.camera())->spatialNode));
     }
+
+    // Ensure these have a parent. All nodes need to be in the node tree somewhere, even if they're technically "parentless"
+    // or we'll not assign a storage slot for them or update them.
+    for (QSSGRenderCamera *camera : std::as_const(layerNode.explicitCameras)) {
+        if (!camera->parent)
+            layerNode.addChild(*camera);
+    }
 }
 
 void QQuick3DViewport::updateSceneManagerForImportScene()
